/*
 * Copyright 2009-2010 by egnite GmbH
 * Copyright 2006-2007 by egnite Software GmbH
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 */

#include "config.h"
#include <arch/arm.h>

        .global _reset
_reset:
        b       _start
        b       _loop
        b       _loop
        b       _loop
        b       _loop
        .word   __bss_start - SRAM0_START
        b       _loop
        b       _loop

_loop:
	b 	_loop

_start:
        /* Test if the main oscillator is running. */
        ldr     r1, =PMC_BASE
        ldr     r0, [r1, #PMC_SR_OFF]
        tst     r0, #PMC_MOSCS
        bne     sel_mainck

        /*
         * If not running, enable the main oscillator. Set startup time
         * of 64 * 8 slow clock cycles and wait until oscillator is stable.
         */
        mov     r0, #(64 << CKGR_OSCOUNT_LSB)
        orr     r0, r0, #CKGR_MOSCEN
        str     r0, [r1, #CKGR_MOR_OFF]
1:
        ldr     r0, [r1, #CKGR_MCFR_OFF]
        tst     r0, #CKGR_MAINRDY
        beq     1b

        /*
         * CKGR_MCFR contains the number of main clock cycles within
         * 16 slow clock cycles. Use this value to test if the main
         * clock is running at higher frequency than the slow clock.
         * If yes, select the main clock.
         */
sel_mainck:
        ldr     r0, [r1, #CKGR_MCFR_OFF]
        mov     r0, r0, asl #16 /* Main clock is in lower 16 bits. */
        mov     r0, r0, lsr #16
        cmp     r0, #16
        bls     set_pll
        mov     r0, #PMC_CSS_MAIN_CLK
        str     r0, [r1, #PMC_MCKR_OFF]

        /*
         * Configure PLL A.
         * PLLfreq = crystal / divider * (multiplier + 1)
         * Wait max clock cycles until PLL is locked.
         */
set_pll:
        ldr     r0, =(_BV(29) | CKGR_OUT_2 | (63 << CKGR_PLLCOUNT_LSB) | (PLL_MUL_VAL << CKGR_MUL_LSB) | (PLL_DIV_VAL << CKGR_DIV_LSB))
        str     r0, [r1, #CKGR_PLLAR_OFF]
1:
        ldr     r0, [r1, #PMC_SR_OFF]
        and     r0, r0, #PMC_LOCKA
        cmp     r0, #0
        beq     1b

        /*
         * Select PLL A for the processor clock (PCK) and set the master
         * clock prescaler and divider.
         */
        ldr     r0, =(PMC_CSS_PLLA_CLK | PMC_PRES_CLK | PMC_MDIV_2)
        str     r0, [r1, #PMC_MCKR_OFF]
1:
        ldr     r0, [r1, #PMC_SR_OFF]
        and     r0, r0, #PMC_MCKRDY
        cmp     r0, #0
        beq     1b

        /* Enable instruction cache. */
        mrc     p15, 0, r0, c1, c0, 0
        orr     r0, r0, #(1 << CP15_CR1_I_BIT)
        mcr     p15, 0, r0, c1, c0, 0

        /* Set the EBI slot cycle limit to 64. */
        ldr     r1, =MATRIX_BASE
        ldr     r0, [r1, #(MATRIX_SCFG_OFF + (4 * MATRIX_SLAVE_EBI))]
        and     r0, r0, #0xffffff00
        orr     r0, r0, #64
        str     r0, [r1, #(MATRIX_SCFG_OFF + (4 * MATRIX_SLAVE_EBI))]

        /* Set ARM mode. */
        msr     CPSR_c, #ARM_MODE_SYS | ARM_CPSR_I_BIT | ARM_CPSR_F_BIT

/*
 * Clear bss and copy data.
 */
        ldr     r1, =__bss_start
        ldr     r2, =__bss_end
        ldr     r3, =0

1:
        cmp     r1, r2
        strne   r3, [r1], #+4
        bne     1b

        /* Jump to C code. */
        ldr     r13,=(SRAM0_START + SRAM0_SIZE)
        ldr     r0, =main
        mov     lr, pc
        bx      r0
