/*
 * Copyright 2010 by egnite GmbH
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 */

#include "config.h"

#include <arch/arm.h>
#include <stdio.h>

#include "at45d.h"
#include "sdram.h"

/*!
 * \brief Load image from DataFlash into SDRAM and run it.
 *
 * \return Always 0.
 */
int main(void)
{
    /* Enable clocks. */
    outr(PMC_PCER, _BV(PIOA_ID) | _BV(PIOC_ID) | _BV(SPI0_ID));

#if UART_OUTPUT
    printf("\nSAMBOOT " VERSION " : ");
#endif

    /* Init SDRAM. */
    SdramInit();

    /* Init DataFlash. */
    if (DataFlashInit()) {

        /* No known DataFlash. */
#if UART_OUTPUT
        puts("Not found");
#endif

    } else {

        /* DataFlash available, check if recover button pressed. */
#if defined(DF_RECOVER_PIO) && defined(DF_RECOVER_BIT)
        outr(DF_RECOVER_PIO + PIO_PUER_OFF, _BV(DF_RECOVER_BIT));
        if ((inr(DF_RECOVER_PIO + PIO_PDSR_OFF) & _BV(DF_RECOVER_BIT)) == 0) {
            /* Button pressed, remove ourself from first flash page. */
            DataFlashRecover();
#if UART_OUTPUT
            puts("SAM-BA enabled");
#endif
        } else
#endif

        {
            uint8_t *sdram_start = (uint8_t *)SDRAM_BASE;

#if UART_OUTPUT
            printf("Loading %#x ", IMAGE_START);
#endif
            /* Transfer image from DataFLash to SDRAM. */
            DataFlashRead(IMAGE_START, sdram_start, IMAGE_END - IMAGE_START);
#if UART_OUTPUT
            puts("OK");
#endif
            /* Run the image. */
            __asm__ __volatile__ ("bx      %0" : : "r" (sdram_start));
        } 
    }
    return 0;
}
