/*
 * Copyright 2010 by egnite GmbH
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 */
#include "config.h"

#include <arch/arm.h>
#include <stdio.h>

#include "utils.h"
#include "sdram.h"

/*!
 * \brief Board specific SDRAM initialization.
 */
void SdramInit(void)
{
#if defined(AT91SAM9260_EK)
    /* Configure the SDRAMC PIO */
    outr(PIOC_ASR, 0xFFFF0000);
    outr(PIOC_PDR, 0xFFFF0000);

    /* Initialize the matrix */
    outr(CCFG_CSA, inr(CCFG_CSA) | CCFG_CS1A);

    outr(SDRAMC_CR, SDRAMC_NC_9  | SDRAMC_NR_13 | SDRAMC_CAS_2 | SDRAMC_NB |
        (2 << SDRAMC_TWR_LSB) | (7 << SDRAMC_TRC_LSB) | (2  << SDRAMC_TRP_LSB) |
        (2 << SDRAMC_TRCD_LSB) | (5 << SDRAMC_TRAS_LSB) | (8 << SDRAMC_TXSR_LSB));
    outr(SDRAMC_MDR, SDRAMC_MD_SDRAM);
    Delay(500);

    outr(SDRAMC_MR, SDRAMC_MODE_NOP);
    outr(SDRAM_BASE, 0);

    outr(SDRAMC_MR, SDRAMC_MODE_PRCGALL);
    outr(SDRAM_BASE, 0);
    Delay(500);

    outr(SDRAMC_MR, SDRAMC_MODE_RFSH);
    outr(SDRAM_BASE + 4, 1);
    outr(SDRAMC_MR, SDRAMC_MODE_RFSH);
    outr(SDRAM_BASE + 8, 2);
    outr(SDRAMC_MR, SDRAMC_MODE_RFSH);
    outr(SDRAM_BASE + 12, 3);
    outr(SDRAMC_MR, SDRAMC_MODE_RFSH);
    outr(SDRAM_BASE + 16, 4);
    outr(SDRAMC_MR, SDRAMC_MODE_RFSH);
    outr(SDRAM_BASE + 20, 5);
    outr(SDRAMC_MR, SDRAMC_MODE_RFSH);
    outr(SDRAM_BASE + 24, 6);
    outr(SDRAMC_MR, SDRAMC_MODE_RFSH);
    outr(SDRAM_BASE + 28, 7);
    outr(SDRAMC_MR, SDRAMC_MODE_RFSH);
    outr(SDRAM_BASE + 32, 8);

	outr(SDRAMC_MR, SDRAMC_MODE_LMR);
	outr(SDRAM_BASE + 36, 0xCAFEDEDE);

	outr(SDRAMC_TR, 696);

	outr(SDRAMC_MR, SDRAMC_MODE_NORMAL);
	outr(SDRAM_BASE, 0);
#endif
}
